<?php


namespace Tygh\Addons\CommerceML\Tests\Unit\Convertors;


use Tygh\Addons\CommerceML\Convertors\OrderConvertor;
use Tygh\Addons\CommerceML\Dto\IdDto;
use Tygh\Addons\CommerceML\Dto\OrderDto;
use Tygh\Addons\CommerceML\Dto\OrderProductDto;
use Tygh\Addons\CommerceML\Dto\TranslatableValueDto;
use Tygh\Addons\CommerceML\Tests\Unit\StorageBasedTestCase;
use Tygh\Addons\CommerceML\Xml\SimpleXmlElement;

class OrderConvertorXmlTest extends StorageBasedTestCase
{
    protected function setUp()
    {
        $this->requireMockFunction('fn_set_hook');

        parent::setUp();
    }

    /**
     * @param $xml
     * @param $expected_entity
     * @dataProvider dpConvert
     */
    public function testConvert($xml, $expected_entity)
    {
        $convertor = new OrderConvertor();
        $import_storage = $this->getImportStorage(null, []);
        $convertor->convert(simplexml_load_string($xml, SimpleXmlElement::class, LIBXML_NOCDATA), $import_storage);
        $this->assertEquals([$expected_entity], $import_storage->getImportEntityRepository()->getEntites());

    }

    public function dpConvert()
    {
        return [
            $this->getImportCase1(),
            $this->getImportCase2()
        ];
    }

    private function getImportCase1()
    {
        $order = new OrderDto();
        $order->id = IdDto::createByLocalId(100);
        $order->id->external_id = '9e13cb41-0fa2-11eb-b011-06aab7dbc213';
        $order->shipping_cost = 28;

        $product = new OrderProductDto();
        $product->id = new IdDto('dee6e19a-55bc-11d9-848a-00112f43529a');
        $product->amount = 3;
        $product->price = 2650;
        $product->total_price = 7950;

        $order->products[] = $product;

        $order->subtotal = 7950;
        $order->status = new TranslatableValueDto('Closed');
        $order->updated_at = strtotime('2020-10-29 10:19:00');

        $xml = <<<XML
<Документ>
	<Ид>9e13cb41-0fa2-11eb-b011-06aab7dbc213</Ид>
	<Номер>100</Номер>
	<Дата>2020-10-29</Дата>
	<ХозОперация>Заказ товара</ХозОперация>
	<Роль>Продавец</Роль>
	<Валюта>840</Валюта>
	<Курс>77.552</Курс>
	<Сумма>3178</Сумма>
	<Контрагенты>
		<Контрагент>
			<Ид>e609b34a-03d9-11eb-b011-06aab7dbc213</Ид>
			<Наименование>Admin Admin</Наименование>
			<ПолноеНаименование>Admin Admin</ПолноеНаименование>
			<Роль>Покупатель</Роль>
		</Контрагент>
	</Контрагенты>
	<Время>10:19:00</Время>
	<Налоги>
		<Налог>
			<Наименование>НДС</Наименование>
			<УчтеноВСумме>false</УчтеноВСумме>
			<Сумма>0</Сумма>
		</Налог>
	</Налоги>
	<Товары>
		<Товар>
			<Ид>3d7b1eac-0403-11eb-b011-06aab7dbc213</Ид>
			<Артикул/>
			<Наименование>Доставка заказа</Наименование>
			<БазоваяЕдиница Код="796" НаименованиеПолное="Штука" МеждународноеСокращение="PCE">
				<Пересчет>
					<Единица>796</Единица>
					<Коэффициент>1</Коэффициент>
				</Пересчет>
			</БазоваяЕдиница>
			<СтавкиНалогов>
				<СтавкаНалога>
					<Наименование>НДС</Наименование>
					<Ставка>0</Ставка>
				</СтавкаНалога>
			</СтавкиНалогов>
			<ЗначенияРеквизитов>
				<ЗначениеРеквизита>
					<Наименование>ТипНоменклатуры</Наименование>
					<Значение>Услуга</Значение>
				</ЗначениеРеквизита>
			</ЗначенияРеквизитов>
			<ЦенаЗаЕдиницу>28</ЦенаЗаЕдиницу>
			<Количество>1</Количество>
			<Сумма>28</Сумма>
			<Единица>796</Единица>
			<Коэффициент>1</Коэффициент>
		</Товар>
		<Товар>
			<Ид>dee6e19a-55bc-11d9-848a-00112f43529a</Ид>
			<Артикул>Т-123456</Артикул>
			<Наименование>0001 Телевизор "JVC"</Наименование>
			<БазоваяЕдиница Код="796" НаименованиеПолное="Штука" МеждународноеСокращение="PCE">
				<Пересчет>
					<Единица>796</Единица>
					<Коэффициент>1</Коэффициент>
				</Пересчет>
			</БазоваяЕдиница>
			<СтавкиНалогов>
				<СтавкаНалога>
					<Наименование>НДС</Наименование>
					<Ставка>0</Ставка>
				</СтавкаНалога>
			</СтавкиНалогов>
			<ЗначенияРеквизитов>
				<ЗначениеРеквизита>
					<Наименование>ТипНоменклатуры</Наименование>
					<Значение>Товар</Значение>
				</ЗначениеРеквизита>
			</ЗначенияРеквизитов>
			<ЦенаЗаЕдиницу>2650</ЦенаЗаЕдиницу>
			<Количество>3</Количество>
			<Сумма>7950</Сумма>
			<Единица>796</Единица>
			<Коэффициент>1</Коэффициент>
		</Товар>
	</Товары>
	<ЗначенияРеквизитов>
		<ЗначениеРеквизита>
			<Наименование>Номер по 1С</Наименование>
			<Значение>ТДЦУ-000017</Значение>
		</ЗначениеРеквизита>
		<ЗначениеРеквизита>
			<Наименование>Дата по 1С</Наименование>
			<Значение>2020-10-29T10:19:00</Значение>
		</ЗначениеРеквизита>
		<ЗначениеРеквизита>
			<Наименование>Проведен</Наименование>
			<Значение>true</Значение>
		</ЗначениеРеквизита>
		<ЗначениеРеквизита>
		    <Наименование>Статус заказа</Наименование>
		    <Значение>Closed</Значение>
        </ЗначениеРеквизита>
	</ЗначенияРеквизитов>
</Документ>
XML;

        return [
            $xml, $order
        ];
    }

    private function getImportCase2()
    {
        $order = new OrderDto();
        $order->id = IdDto::createByLocalId(101);
        $order->id->external_id = '9e13cb41-0fa2-11eb-b011-06aab7dbc214';
        $order->shipping_cost = 28;

        $product = new OrderProductDto();
        $product->id = new IdDto('dee6e19a-55bc-11d9-848a-00112f43529a');
        $product->amount = 3;
        $product->price = 2650;
        $product->total_price = 7950;
        $order->products[] = $product;

        $product2 = new OrderProductDto();
        $product2->id = new IdDto('dee6e19a-55bc-11d9-848a-00112f43529a');
        $product2->amount = 1;
        $product2->price = 2000;
        $product2->total_price = 2000;
        $order->products[] = $product2;

        $order->subtotal = 9950;
        $order->status = new TranslatableValueDto('Open');
        $order->updated_at = strtotime('2020-10-29 10:19:00');

        $xml = <<<XML
<Документ>
	<Ид>9e13cb41-0fa2-11eb-b011-06aab7dbc214</Ид>
	<Номер>101</Номер>
	<Дата>2020-10-29</Дата>
	<ХозОперация>Заказ товара</ХозОперация>
	<Роль>Продавец</Роль>
	<Валюта>840</Валюта>
	<Курс>77.552</Курс>
	<Сумма>3178</Сумма>
	<Контрагенты>
		<Контрагент>
			<Ид>e609b34a-03d9-11eb-b011-06aab7dbc213</Ид>
			<Наименование>Admin Admin</Наименование>
			<ПолноеНаименование>Admin Admin</ПолноеНаименование>
			<Роль>Покупатель</Роль>
		</Контрагент>
	</Контрагенты>
	<Время>10:19:00</Время>
	<Налоги>
		<Налог>
			<Наименование>НДС</Наименование>
			<УчтеноВСумме>false</УчтеноВСумме>
			<Сумма>0</Сумма>
		</Налог>
	</Налоги>
	<Товары>
		<Товар>
			<Ид>3d7b1eac-0403-11eb-b011-06aab7dbc213</Ид>
			<Артикул/>
			<Наименование>Доставка заказа</Наименование>
			<БазоваяЕдиница Код="796" НаименованиеПолное="Штука" МеждународноеСокращение="PCE">
				<Пересчет>
					<Единица>796</Единица>
					<Коэффициент>1</Коэффициент>
				</Пересчет>
			</БазоваяЕдиница>
			<СтавкиНалогов>
				<СтавкаНалога>
					<Наименование>НДС</Наименование>
					<Ставка>0</Ставка>
				</СтавкаНалога>
			</СтавкиНалогов>
			<ЗначенияРеквизитов>
				<ЗначениеРеквизита>
					<Наименование>ТипНоменклатуры</Наименование>
					<Значение>Услуга</Значение>
				</ЗначениеРеквизита>
			</ЗначенияРеквизитов>
			<ЦенаЗаЕдиницу>28</ЦенаЗаЕдиницу>
			<Количество>1</Количество>
			<Сумма>28</Сумма>
			<Единица>796</Единица>
			<Коэффициент>1</Коэффициент>
		</Товар>
		<Товар>
			<Ид>dee6e19a-55bc-11d9-848a-00112f43529a</Ид>
			<Артикул>Т-123456</Артикул>
			<Наименование>0001 Телевизор "JVC"</Наименование>
			<БазоваяЕдиница Код="796" НаименованиеПолное="Штука" МеждународноеСокращение="PCE">
				<Пересчет>
					<Единица>796</Единица>
					<Коэффициент>1</Коэффициент>
				</Пересчет>
			</БазоваяЕдиница>
			<СтавкиНалогов>
				<СтавкаНалога>
					<Наименование>НДС</Наименование>
					<Ставка>0</Ставка>
				</СтавкаНалога>
			</СтавкиНалогов>
			<ЗначенияРеквизитов>
				<ЗначениеРеквизита>
					<Наименование>ТипНоменклатуры</Наименование>
					<Значение>Товар</Значение>
				</ЗначениеРеквизита>
			</ЗначенияРеквизитов>
			<ЦенаЗаЕдиницу>2650</ЦенаЗаЕдиницу>
			<Количество>3</Количество>
			<Сумма>7950</Сумма>
			<Единица>796</Единица>
			<Коэффициент>1</Коэффициент>
		</Товар>
		<Товар>
			<Ид>dee6e19a-55bc-11d9-848a-00112f43529a</Ид>
			<Артикул>Т-123456</Артикул>
			<Наименование>0001 Телевизор "JVC"</Наименование>
			<БазоваяЕдиница Код="796" НаименованиеПолное="Штука" МеждународноеСокращение="PCE">
				<Пересчет>
					<Единица>796</Единица>
					<Коэффициент>1</Коэффициент>
				</Пересчет>
			</БазоваяЕдиница>
			<СтавкиНалогов>
				<СтавкаНалога>
					<Наименование>НДС</Наименование>
					<Ставка>0</Ставка>
				</СтавкаНалога>
			</СтавкиНалогов>
			<ЗначенияРеквизитов>
				<ЗначениеРеквизита>
					<Наименование>ТипНоменклатуры</Наименование>
					<Значение>Товар</Значение>
				</ЗначениеРеквизита>
			</ЗначенияРеквизитов>
			<ЦенаЗаЕдиницу>2000</ЦенаЗаЕдиницу>
			<Количество>1</Количество>
			<Сумма>2000</Сумма>
			<Единица>796</Единица>
			<Коэффициент>1</Коэффициент>
		</Товар>
	</Товары>
	<ЗначенияРеквизитов>
		<ЗначениеРеквизита>
			<Наименование>Номер по 1С</Наименование>
			<Значение>ТДЦУ-000017</Значение>
		</ЗначениеРеквизита>
		<ЗначениеРеквизита>
			<Наименование>Дата по 1С</Наименование>
			<Значение>2020-10-29T10:19:00</Значение>
		</ЗначениеРеквизита>
		<ЗначениеРеквизита>
			<Наименование>Проведен</Наименование>
			<Значение>true</Значение>
		</ЗначениеРеквизита>
		<ЗначениеРеквизита>
		    <Наименование>Статус заказа</Наименование>
		    <Значение>Open</Значение>
        </ЗначениеРеквизита>
	</ЗначенияРеквизитов>
</Документ>
XML;

        return [
            $xml, $order
        ];
    }
}